@@ -1,6 +1,9 @@
 """
 Implementations of algorithms from various specifications.
 
+- :func:`.remove_url_auth`: removes and returns the auth portion
+  of a URL.  This is particularly handy for processing URLs from
+  configuration files or environment variables.
 - :func:`.rewrite_url`: modify a portion of a URL.
 - :func:`.select_content_type`: select the best match between a
   HTTP ``Accept`` header and a list of available ``Content-Type`` s
@@ -15,13 +18,12 @@ described in IETF RFCs.
 """
 from __future__ import unicode_literals
 from operator import attrgetter
+import collections
 
-from . compat import parse
-from . import errors
+from ietfparse.compat import parse
+from ietfparse import errors
 
-
-IDNA_SCHEMES = [
-    'http', 'https', 'ftp', 'afp', 'sftp', 'smb']
+IDNA_SCHEMES = ['http', 'https', 'ftp', 'afp', 'sftp', 'smb']
 
 # these are in addition to the "always safe" set
 _rfc3986_unreserved = b'~'
@@ -40,15 +42,27 @@ PATH_SAFE_CHARS = _rfc3986_sub_delims + _rfc3986_unreserved + b':/@'
 FRAGMENT_SAFE_CHARS = b'?/'
 
 
+class RemoveUrlAuthResult(
+        collections.namedtuple('RemoveUrlAuthResult', ['auth', 'url'])):
+    __slots__ = ()
+
+    @property
+    def username(self):
+        return self.auth[0]
+
+    @property
+    def password(self):
+        return self.auth[1]
+
+
 def _content_type_matches(candidate, pattern):
     """Is ``candidate`` an exact match or sub-type of ``pattern``?"""
     def _wildcard_compare(type_spec, type_pattern):
         return type_pattern == '*' or type_spec == type_pattern
 
-    return (
-        _wildcard_compare(candidate.content_type, pattern.content_type) and
-        _wildcard_compare(candidate.content_subtype, pattern.content_subtype)
-    )
+    return (_wildcard_compare(candidate.content_type, pattern.content_type)
+            and _wildcard_compare(candidate.content_subtype,
+                                  pattern.content_subtype))
 
 
 def select_content_type(requested, available):
@@ -72,11 +86,10 @@ def select_content_type(requested, available):
     producing.  The selected value should ultimately be used as the
     `Content-Type`_ header in the generated response.
 
-    .. _Accept: http://tools.ietf.org/html/rfc7231#section-5.3.2
-    .. _Content-Type: http://tools.ietf.org/html/rfc7231#section-3.1.1.5
+    .. _Accept: https://tools.ietf.org/html/rfc7231#section-5.3.2
+    .. _Content-Type: https://tools.ietf.org/html/rfc7231#section-3.1.1.5
 
     """
-
     class Match(object):
         """Sorting assistant.
 
@@ -126,12 +139,15 @@ def select_content_type(requested, available):
                     else:
                         self.parameter_distance += 1
 
+    def extract_quality(obj):
+        return getattr(obj, 'quality', 1.0)
+
     matches = []
-    for pattern in sorted(requested, key=attrgetter('quality'), reverse=True):
+    for pattern in sorted(requested, key=extract_quality, reverse=True):
         for candidate in sorted(available):
             if _content_type_matches(candidate, pattern):
                 if candidate == pattern:  # exact match!!!
-                    if pattern.quality == 0.0:
+                    if extract_quality(pattern) == 0.0:
                         raise errors.NoMatch  # quality of 0 means NO
                     return candidate, pattern
                 matches.append(Match(candidate, pattern))
@@ -215,25 +231,27 @@ def rewrite_url(input_url, **kwargs):
     pass through case that is almost always present.
 
     """
-    scheme, netloc, path, query, fragment = parse.urlsplit(input_url)
+    result = parse.urlparse(input_url)
 
     if 'scheme' in kwargs:
         scheme = kwargs['scheme']
+    else:
+        scheme = result.scheme
 
-    ident, host_n_port = parse.splituser(netloc)
-
-    user, password = parse.splitpasswd(ident) if ident else (None, None)
+    user = None
     if 'user' in kwargs:
         user = kwargs['user']
-    elif user is not None:
-        user = parse.unquote_to_bytes(user).decode('utf-8')
+    elif result.username is not None:
+        user = parse.unquote_to_bytes(result.username).decode('utf-8')
+
+    password = None
     if 'password' in kwargs:
         password = kwargs['password']
-    elif password is not None:
-        password = parse.unquote_to_bytes(password).decode('utf-8')
+    elif result.password is not None:
+        password = parse.unquote_to_bytes(result.password).decode('utf-8')
+
     ident = _create_url_identifier(user, password)
 
-    host, port = parse.splitnport(host_n_port, defport=None)
     if 'host' in kwargs:
         host = kwargs['host']
         if host is not None:
@@ -243,13 +261,17 @@ def rewrite_url(input_url, **kwargs):
                 encode_with_idna=kwargs.get('encode_with_idna', None),
                 scheme=scheme,
             )
+    else:
+        host = result.hostname
 
     if 'port' in kwargs:
         port = kwargs['port']
         if port is not None:
             port = int(kwargs['port'])
             if port < 0:
-                raise ValueError('port is required to be non-negative')
+                raise ValueError('port is requried to be non-negative')
+    else:
+        port = result.port
 
     if host is None or host == '':
         host_n_port = None
@@ -264,6 +286,8 @@ def rewrite_url(input_url, **kwargs):
             path = '/'
         else:
             path = parse.quote(path.encode('utf-8'), safe=PATH_SAFE_CHARS)
+    else:
+        path = result.path
 
     netloc = '{0}@{1}'.format(ident, host_n_port) if ident else host_n_port
 
@@ -289,12 +313,16 @@ def rewrite_url(input_url, **kwargs):
                 query = parse.urlencode(params)
             else:
                 query = new_query
+    else:
+        query = result.query
 
     if 'fragment' in kwargs:
         fragment = kwargs['fragment']
         if fragment is not None:
             fragment = parse.quote(fragment.encode('utf-8'),
                                    safe=FRAGMENT_SAFE_CHARS)
+    else:
+        fragment = result.fragment
 
     # The following is necessary to get around some interesting special
     # case code in urllib.parse._coerce_args in Python 3.4.  Setting
@@ -303,7 +331,45 @@ def rewrite_url(input_url, **kwargs):
     if scheme is None:
         scheme = ''
 
-    return parse.urlunsplit((scheme, netloc, path, query, fragment))
+    return parse.urlunparse(
+        (scheme, netloc, path, result.params, query, fragment))
+
+
+def remove_url_auth(url):
+    """
+    Removes the user & password and returns them along with a new url.
+
+    :param str url: the URL to sanitize
+    :return: a :class:`tuple` containing the authorization portion and
+        the sanitized URL.  The authorization is a simple user & password
+        :class:`tuple`.
+
+    >>> auth, sanitized = remove_url_auth('http://foo:bar@example.com')
+    >>> auth
+    ('foo', 'bar')
+    >>> sanitized
+    'http://example.com'
+
+    The return value from this function is simple named tuple with the
+    following fields:
+
+    - *auth* the username and password as a tuple
+    - *username* the username portion of the URL or :data:`None`
+    - *password* the password portion of the URL or :data:`None`
+    - *url* the sanitized URL
+
+    >>> result = remove_url_auth('http://me:secret@example.com')
+    >>> result.username
+    'me'
+    >>> result.password
+    'secret'
+    >>> result.url
+    'http://example.com'
+
+    """
+    parts = parse.urlparse(url)
+    return RemoveUrlAuthResult(auth=(parts.username or None, parts.password),
+                               url=rewrite_url(url, user=None, password=None))
 
 
 def _create_url_identifier(user, password):
@@ -324,7 +390,9 @@ def _create_url_identifier(user, password):
     return None
 
 
-def _normalize_host(host, enable_long_host=False, encode_with_idna=None,
+def _normalize_host(host,
+                    enable_long_host=False,
+                    encode_with_idna=None,
                     scheme=None):
     """
     Normalize a host for a URL.
@@ -354,8 +422,8 @@ def _normalize_host(host, enable_long_host=False, encode_with_idna=None,
         enable_idna = scheme.lower() in IDNA_SCHEMES if scheme else False
     if enable_idna:
         try:
-            host = '.'.join(segment.encode('idna').decode()
-                            for segment in host.split('.'))
+            host = '.'.join(
+                segment.encode('idna').decode() for segment in host.split('.'))
         except UnicodeError as exc:
             raise ValueError('host is invalid - {0}'.format(exc))
     else:
